@@ -11,16 +11,20 @@ use CGI::Carp qw(fatalsToBrowser);
 use CVS::Metrics;
 use CVS::Metrics::Graph;
 
-my $cvs_root = param("cvsroot");
+my $extract = param("extract");
+my $repository = param("repository");
+my $module = param("module");
 my $path = param("path");
 my $tag_from = param("from_tag");
 my $tag_to = param("to_tag");
+my $force = param("force");
 
+my $cvs_root = $extract . "/" . $repository . "/" . $module;
 chdir $cvs_root
 		or die "can't change dir $cvs_root ($!).\n";
 
 my $cfg = ".cvs_metrics";
-our ($title, $regex_tag, $start_date);
+our ($title, $regex_tag, $start_date, $regex_ignore_tag);
 if ( -r $cfg) {
 	warn "reading $cfg\n";
 	require $cfg;
@@ -31,7 +35,7 @@ unless (defined $regex_tag) {
 }
 
 unless (defined $start_date) {
-	$start_date = "2003/01/01";
+	$start_date = "2004/01/01";
 }
 
 my $cvs = FindCvs();
@@ -121,14 +125,15 @@ Francois PERRAD, francois.perrad@gadz.org
 our $cvs_log = CVS::Metrics::CvsLog(
 		stream		=> $cvs_logfile,
 		use_cache	=> 1,
+		force		=> $force,
 );
 if ($cvs_log) {
 	our @tags;
-	my $timed = $cvs_log->getTimedTag();
+	my $timed = $cvs_log->getTimedTag($regex_ignore_tag);
 	my %matched;
 	while (my ($tag, $date) = each %{$timed}) {
 		if ($tag =~ /$regex_tag/) {
-			$matched{$date} = $tag;
+			$matched{$date.$tag} = $tag;
 		}
 	}
 	foreach (sort keys %matched) {
@@ -213,6 +218,7 @@ my $html = q{
     <meta http-equiv='Content-Type' content='text/html; charset=ISO-8859-1' />
     <meta name='generator' content='<TMPL_VAR NAME=generator>' />
     <meta name='date' content='<TMPL_VAR NAME=date>' />
+    <meta name='robots' content='nofollow' />
     <title>cvs_current <!-- TMPL_VAR NAME=title --></title>
     <style type='text/css'>
       <!-- TMPL_VAR NAME=style -->
@@ -270,7 +276,7 @@ my $html = q{
   </table>
   <hr />
   <h2>Detailed Evolution Report</h2>
-  <table border='1' cellpadding='5'>
+  <table width='100%' border='1' cellpadding='5'>
     <tr>
       <th width='20%'>Directories</th>
       <th width='40%'>Messages</th>
@@ -314,12 +320,14 @@ my $style = q{
       table.layout  { background-color: #CCFFFF }
       span.author   { font-weight: bold }
       span.filename { color: blue }
+      a.filename    { font-weight: normal }
       span.revision { font-weight: bold; color: blue }
       span.tag      { font-weight: bold }
       span.date     { }
       span.deleted  { font-weight: bold; color: red }
       span.added    { font-weight: bold; color: blue }
       span.modified { font-weight: bold }
+      a.modified    { font-weight: bold }
 };
 
 	$html =~ s/^\s+//gm;
@@ -349,7 +357,7 @@ my $style = q{
 		close OUT;
 	}
 
-	my $timed_tag = $cvs_log->getTimedTag();
+	my $timed_tag = $cvs_log->getTimedTag($regex_ignore_tag);
 	my @timed_tag = ();
 	foreach my $tag (@{$tags}) {
 		if ($tag eq "HEAD") {
@@ -430,11 +438,20 @@ my $style = q{
 					if ($file->{revision} =~ /^1(\.1)+$/) {
 						$action = "<span class='added'>ADDED</span>";
 					} else {
-						$action = "<span class='modified'>MODIFIED</span>";
+						my $r1 = $file->{revision};
+						$r1 =~ s/(\d+)$/$1-1/e; 
+						my $r2 = $file->{revision};
+						$action = "<a class='modified' href='cvsweb.cgi/" . $module . "/" . $file->{filename} . ".diff?r1=" . $r1 . "\&r2=" . $r2 . "\&cvsroot=" . $repository . "'>MODIFIED</a>";
 					}
 				}
+				my $filename;
+				if ($file->{state} eq 'dead') {
+					$filename = basename($file->{filename});
+				} else {
+					$filename = "<a class='filename' href='cvsweb.cgi/" . $module . "/" . $file->{filename} . "?rev=" . $file->{revision} . "\&content-type=text\%2Fplain\&cvsroot=" . $repository . "'>" . basename($file->{filename}) . "</a>"; 
+				}
 				push @files, {
-					filename	=> basename($file->{filename}),
+					filename	=> $filename,
 					revision	=> $file->{revision},
 					date		=> $file->{date},
 					author		=> $file->{author},
@@ -504,7 +521,7 @@ my $error = q{
     <h2>Error</h2>
     <blockquote><em><!-- TMPL_VAR NAME=message --></em></blockquote>
     <hr />
-    <cite>Generated by cvs_cgilog (<!-- TMPL_VAR NAME=date -->)</cite>
+    <cite>Generated by cgi_cvs_evolr (<!-- TMPL_VAR NAME=date -->)</cite>
   </body>
 </html>
 };
