@@ -26,7 +26,7 @@ if ($opts{v}) {
 }
 
 my $cfg = ".cvs_metrics";
-our ($title, $regex_tag, @dirs, $flg_css, $start_date);
+our ($title, $regex_tag, @dirs, $flg_css, $start_date, $regex_branch, $regex_ignore_tag);
 if ( -r $cfg) {
 	print "reading $cfg\n";
 	require $cfg;
@@ -59,10 +59,14 @@ unless (defined $regex_tag) {
 	$regex_tag = '\d+';
 }
 
+unless (defined $regex_branch) {
+	$regex_branch = '^BRANCH';
+}
+
 if ($opts{S}) {
 	$start_date = $opts{S};
 } else {
-	$start_date = "2003/01/01" unless (defined $start_date);
+	$start_date = "2004/01/01" unless (defined $start_date);
 }
 
 my $output = $opts{o};
@@ -148,6 +152,8 @@ in the current directory. The file could contains the following variables :
 
  $regex_tag = '^V\d+';
 
+ $regex_tag = '^BRANCH';
+
  @dirs = ( "abc", "def" , "def/hij" );
 
  $start_date = "2003/01/01";
@@ -158,7 +164,7 @@ cvs_activity, cvs_energy, cvs_tklog, cvs_wxlog
 
 =head1 COPYRIGHT
 
-(c) 2004 Francois PERRAD, France. All rights reserved.
+(c) 2004-2005 Francois PERRAD, France. All rights reserved.
 
 This library is distributed under the terms of the Artistic Licence.
 
@@ -175,12 +181,12 @@ our $cvs_log = CVS::Metrics::CvsLog(
 );
 if ($cvs_log) {
 	our @tags;
-	my $timed = $cvs_log->getTimedTag();
+	my $timed = $cvs_log->getTimedTag($regex_ignore_tag);
 	my %matched;
 	while (my ($tag, $date) = each %{$timed}) {
 		print "Tag: ", $tag;
 		if ($tag =~ /$regex_tag/) {
-			$matched{$date} = $tag;
+			$matched{$date.$tag} = $tag;
 			print " ... matched";
 		}
 		print "\n";
@@ -188,17 +194,28 @@ if ($cvs_log) {
 	foreach (sort keys %matched) {
 		push @tags, $matched{$_};
 	}
+	our @branchs;
+	foreach ($cvs_log->getTagname()) {
+		if (/$regex_branch/) {
+			push @branchs, $_;
+		}
+	}
+	print "Branch : @branchs\n";
 
 	my $tag_from = $tags[-1];
 	push @tags, "HEAD";
 	$cvs_log->insertHead();
 
-	my @html = ();
+	my @html = ();                                                                              
 	push @html, GenerateHTML($cvs_log, \@tags, $title, ".", $tag_from, "HEAD", $flg_css, $start_date, $output);
 	for my $path (@dirs) {
 		push @html, GenerateHTML($cvs_log, \@tags, $title, $path, $tag_from, "HEAD", $flg_css, $start_date, $output);
 	}
-	GenerateSummary($title, $flg_css, \@html, $output);
+	my @html2 = ();                                                                              
+	for my $branch (@branchs) {
+		push @html2, GenerateBranchHTML($cvs_log, $branch, $title, ".", $flg_css, $output);
+	}
+	GenerateSummary($title, $flg_css, \@html, \@html2, $output);
 }
 
 sub FindCvs {
@@ -229,7 +246,7 @@ sub FindCvs {
 #######################################################################
 
 sub GenerateHTML {
-	my ($cvs_log, $tags, $title, $path, $tag_from, $tag_to, $flg_css, $start_date) = @_;
+	my ($cvs_log, $tags, $title, $path, $tag_from, $tag_to, $flg_css, $start_date, $output) = @_;
 
 my $html = q{
 <?xml version='1.0' encoding='ISO-8859-1'?>
@@ -377,7 +394,7 @@ my $style = q{
 		close OUT;
 	}
 
-	my $timed_tag = $cvs_log->getTimedTag();
+	my $timed_tag = $cvs_log->getTimedTag($regex_ignore_tag);
 	my @timed_tag = ();
 	foreach my $tag (@{$tags}) {
 		if ($tag eq "HEAD") {
@@ -529,8 +546,213 @@ my $style = q{
 	return $basename;
 }
 
+sub GenerateBranchHTML {
+	my ($cvs_log, $branch, $title, $path, $flg_css, $output) = @_;
+
+my $html = q{
+<?xml version='1.0' encoding='ISO-8859-1'?>
+<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Transitional//EN' 'http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd'>
+<html xmlns='http://www.w3.org/1999/xhtml'>
+  <head>
+    <meta http-equiv='Content-Type' content='text/html; charset=ISO-8859-1' />
+    <meta name='generator' content='<TMPL_VAR NAME=generator>' />
+    <meta name='date' content='<TMPL_VAR NAME=date>' />
+    <title>cvs_current <!-- TMPL_VAR NAME=title --></title>
+    <!-- TMPL_IF NAME=css -->
+    <link href='cvs_current.css' rel='stylesheet' type='text/css'/>
+    <!-- TMPL_ELSE -->
+    <style type='text/css'>
+      <!-- TMPL_VAR NAME=style -->
+    </style>
+    <!-- /TMPL_IF -->
+  </head>
+  <body>
+  <h1>Branch Report</h1>
+  <h1><!-- TMPL_VAR NAME=title --></h1>
+  <hr />
+  <h2>Branch Report Summary</h2>
+  <table border='1' cellpadding='5'>
+    <tr>
+      <th>Directories</th>
+      <th># files</th>
+    </tr>
+  <!-- TMPL_LOOP NAME=summary -->
+    <tr>
+      <td><a href='#<TMPL_VAR NAME=dir>'><!-- TMPL_VAR NAME=dir --></a></td>
+      <td><!-- TMPL_VAR NAME=nb --></td>
+    </tr>
+  <!-- /TMPL_LOOP -->
+    <tr class='total'>
+      <td>TOTAL</td>
+      <td><!-- TMPL_VAR NAME=total --></td>
+    </tr>
+  </table>
+  <hr />
+  <h2>Branch Evolution Report</h2>
+  <table border='1' cellpadding='5'>
+    <tr>
+      <th width='20%'>Directories</th>
+      <th width='40%'>Messages</th>
+      <th width='30%'>File Descriptions</th>
+      <th width='10%'>Actions</th>
+    </tr>
+  <!-- TMPL_LOOP NAME=dirs --><tr>
+      <td valign='top' rowspan='<TMPL_VAR NAME=rowspan>'><a id='<TMPL_VAR NAME=dir>'
+                 name='<TMPL_VAR NAME=dir>' /><!-- TMPL_VAR NAME=dir --></td>
+    <!-- TMPL_LOOP NAME=comments -->
+      <!-- TMPL_UNLESS NAME=__FIRST__ --><tr><!-- /TMPL_UNLESS -->
+      <td valign='top' rowspan='<TMPL_VAR NAME=rowspan>'><!-- TMPL_VAR NAME=comment --></td>
+      <!-- TMPL_LOOP NAME=files -->
+        <!-- TMPL_UNLESS NAME=__FIRST__ --><tr><!-- /TMPL_UNLESS -->
+        <td><span class='filename'><!-- TMPL_VAR NAME=filename --></span>
+            <span class='revision'><!-- TMPL_VAR NAME=revision --></span><br />
+          <!-- TMPL_LOOP NAME=tags -->
+            <span class='tag'><!-- TMPL_VAR NAME=tag --></span><br />
+          <!-- /TMPL_LOOP -->
+            <span class='author'><!-- TMPL_VAR NAME=author --></span>
+            <span class='date'><!-- TMPL_VAR NAME=date --></span></td>
+        <td><!-- TMPL_VAR NAME=action --></td>
+      </tr><!-- /TMPL_LOOP -->
+    <!-- /TMPL_LOOP -->
+  <!-- /TMPL_LOOP -->
+  </table>
+  <hr />
+  <cite>Generated by cvs_current (<!-- TMPL_VAR NAME=date -->)</cite>
+  </body>
+</html>
+};
+
+my $style = q{
+      body  { background-color: #FFFFCC }
+      table { background-color: #FFFFFF }
+      th    { background-color: #DCDCDC }
+      h1    { text-align: center }
+      h2    { color: red }
+      td a  { font-weight: bold }
+      tr.total      { font-weight: bold }
+      table.layout  { background-color: #FFFFCC }
+      span.author   { font-weight: bold }
+      span.filename { color: blue }
+      span.revision { font-weight: bold; color: blue }
+      span.tag      { font-weight: bold }
+      span.date     { }
+      span.deleted  { font-weight: bold; color: red }
+      span.added    { font-weight: bold; color: blue }
+      span.modified { font-weight: bold }
+};
+
+	my $template = new HTML::Template(
+			loop_context_vars	=> 1,
+			scalarref			=> \$html,
+	);
+	die "can't create template ($!).\n"
+			unless (defined $template);
+
+	my $now = localtime();
+	my $generator = "cvs_current " . $CVS::Metrics::VERSION . " (Perl " . $] . ")";
+	my $title_full = "${title} ${branch}";
+
+
+	my $dir_evol = $cvs_log->getDirBranch($path, $branch);
+	my @summary = ();
+	my $total = 0; 
+	foreach my $dirname (sort keys %{$dir_evol}) {
+		my $nb = $dir_evol->{$dirname};
+		next unless ($nb);
+		push @summary, {
+			dir		=> $dirname,
+			nb		=> $nb,
+		};
+		$total += $nb; 
+	}
+
+	my $evol = $cvs_log->getBranch($path, $branch);
+	my @dirs = ();
+	foreach my $dirname (sort keys %{$evol}) {
+		my $dir = $evol->{$dirname};
+		my %date_sorted;
+		next unless (scalar keys %{$dir});
+		foreach my $message (keys %{$dir}) {
+			my $files = $dir->{$message};
+			my $file0 = $files->[0];
+			$date_sorted{$file0->{date}} = $message;
+		}
+		my $rowspan1 = 0;
+		my @comments = ();
+		foreach (sort keys %date_sorted) {
+			my $message = $date_sorted{$_};
+			my $files = $dir->{$message};
+			my $rowspan2 = 0;
+			my @files = ();
+			foreach my $file (@{$files}) {
+				my @tags = ();
+				foreach my $tag (sort @{$file->{tags}}) {
+					push @tags, {
+						tag			=> $tag,
+					};
+				}
+				my $action;
+				if ($file->{state} eq 'dead') {
+					$action = "<span class='deleted'>DELETED</span>";
+				} else {
+					if (exists $file->{orig} and ${$file->{orig}}[1] eq "dead") {
+						$action = "<span class='added'>ADDED</span>";
+					} else {
+						$action = "<span class='modified'>MODIFIED</span>";
+					}
+				}
+				push @files, {
+					filename	=> basename($file->{filename}),
+					revision	=> $file->{revision},
+					date		=> $file->{date},
+					author		=> $file->{author},
+					action		=> $action,
+					tags		=> \@tags,
+				};
+				$rowspan1 ++;
+				$rowspan2 ++;
+			}
+			$message =~ s/&/&amp;/g;
+			$message =~ s/</&lt;/g;
+			$message =~ s/>/&gt;/g;
+			$message =~ s/\n/<br \/>/g;
+			push @comments, {
+				rowspan		=> $rowspan2,
+				comment		=> $message,
+				files		=> \@files,
+			};
+		}
+		push @dirs, {
+			rowspan		=> $rowspan1,
+			dir			=> $dirname,
+			comments	=> \@comments,
+		}
+	}
+
+	$template->param(
+			css				=> $flg_css,
+			style			=> $style,
+			generator		=> $generator,
+			date			=> $now,
+			title			=> $title_full,
+			summary			=> \@summary,
+			total			=> $total, 
+			dirs			=> \@dirs,
+	);
+
+	my $basename = "${title_full}.html";
+	$basename =~ s/[ \/]/_/g;
+	my $filename = (defined $output) ? $output . "/" . $basename : $basename;
+	open OUT, "> $filename"
+			or die "can't open $filename ($!)\n";
+	print OUT $template->output();
+	close OUT;
+
+	return $basename;
+}
+
 sub GenerateSummary {
-	my ($title, $flg_css, $r_html, $output) = @_;
+	my ($title, $flg_css, $r_html, $r_html2, $output) = @_;
 
 my $html = q{
 <?xml version='1.0' encoding='ISO-8859-1'?>
@@ -558,6 +780,14 @@ my $html = q{
     <li><a href='<TMPL_VAR NAME=href>'><!-- TMPL_VAR NAME=href --></a></li>
   <!-- /TMPL_LOOP -->
   </ul>
+  <!-- TMPL_IF NAME=branch -->
+  <h2>Branch Reports Index</h2>
+  <ul>
+  <!-- TMPL_LOOP NAME=rpts2 -->
+    <li><a href='<TMPL_VAR NAME=href>'><!-- TMPL_VAR NAME=href --></a></li>
+  <!-- /TMPL_LOOP -->
+  </ul>
+  <!-- /TMPL_IF -->
   <hr />
   <cite>Generated by cvs_current (<!-- TMPL_VAR NAME=date -->)</cite>
   </body>
@@ -586,6 +816,12 @@ my $style = q{
 			href		=> $_,
 		}
 	}
+	my @rpts2 = ();
+	foreach (@{$r_html2}) {
+		push @rpts2, {
+			href		=> $_,
+		}
+	}
 
 	$template->param(
 			css			=> $flg_css,
@@ -594,6 +830,8 @@ my $style = q{
 			date		=> $now,
 			title		=> $title,
 			rpts		=> \@rpts,
+			branch		=> scalar(@{$r_html2}),
+			rpts2		=> \@rpts2,
 	);
 
 	my $filename = "current.html";
