@@ -26,7 +26,7 @@ if ($opts{v}) {
 }
 
 my $cfg = ".cvs_metrics";
-our ($title, $regex_tag, $flg_head, $flg_dead, $flg_css, $start_date);
+our ($title, $regex_tag, $flg_head, $flg_dead, $flg_css, $start_date, $regex_ignore_tag);
 if ( -r $cfg) {
 	print "reading $cfg\n";
 	require $cfg;
@@ -178,12 +178,12 @@ our $cvs_log = CVS::Metrics::CvsLog(
 );
 if ($cvs_log) {
 	our @tags;
-	my $timed = $cvs_log->getTimedTag();
+	my $timed = $cvs_log->getTimedTag($regex_ignore_tag);
 	my %matched;
 	while (my ($tag, $date) = each %{$timed}) {
 		print "Tag: ", $tag;
 		if ($tag =~ /$regex_tag/) {
-			$matched{$date} = $tag;
+			$matched{$date.$tag} = $tag;
 			print " ... matched";
 		}
 		print "\n";
@@ -674,6 +674,7 @@ sub new {
 			flg_css		=> $main::flg_css,
 			start_date	=> $main::start_date,
 			output		=> $main::output,
+			regex_ignore_tag	=> $main::regex_ignore_tag,
 	};
 	bless($self, $class);
 	$self->{path} = '.';
@@ -834,7 +835,7 @@ my $style = q{
 		close OUT;
 	}
 
-	my $timed_tag = $self->{cvs_log}->getTimedTag();
+	my $timed_tag = $self->{cvs_log}->getTimedTag($self->{regex_ignore_tag);
 	my @timed_tag = ();
 	foreach my $tag (@{$self->{tags}}) {
 		if ($tag eq "HEAD") {
